<?php

namespace AlbisPayment5;

use AlbisPayment5\Components\Api\AlbisClient;
use Doctrine\Common\Collections\ArrayCollection;
use Enlight_Event_EventArgs;
use Shopware\Components\Plugin;
use Shopware\Components\Plugin\Context\ActivateContext;
use Shopware\Components\Plugin\Context\DeactivateContext;
use Shopware\Components\Plugin\Context\InstallContext;
use Shopware\Components\Plugin\Context\UninstallContext;
use Shopware\Components\Theme\LessDefinition;

class AlbisPayment5 extends Plugin
{
    public function install(InstallContext $context)
    {
        /** @var \Shopware\Components\Plugin\PaymentInstaller $installer */
        $installer = $this->container->get('shopware.plugin_payment_installer');

        $options = [
            'name' => 'albis_payment',
            'description' => 'Albis Leasing',
            'action' => 'AlbisPayment',
            'active' => 0,
            'position' => 0,
            'additionalDescription' => '<div class="albis-leasing-icon"></div>
                <div id="albis-leasing-description">Sie planen eine Investition? Dann leasen Sie doch,
                   anstatt zu kaufen. Sie schonen damit Ihre Liquidität und können Ihre Anschaffung bei geringer monatlicher
                   Belastung über einen flexiblen Zeitraum hinweg finanzieren.
                </div>',
        ];
        $installer->createOrUpdate($context->getPlugin()->getName(), $options);

        $this->container->get('shopware_attribute.crud_service')->update(
            's_articles_attributes',
            'albis_contract_type',
            'combobox',
            [
                'label' => 'Albis-Vertragsart',
                'displayInBackend' => true,
                'supportText' => '',
                'helpText' => '',
                'position' => 113,
                'arrayStore' => [
                    ['key' => 1, 'value' => 'Vertrag 1'],
                    ['key' => 2, 'value' => 'Vertrag 2']
                ],

            ]
        );

        $this->container->get('db')->executeQuery(
            'INSERT IGNORE INTO s_core_config_mails 
                    VALUE
                    (null, 
                     null,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,
                     ?,                     
                     ?
            )',
            [
                "sORDERAlbis",
                "{config name=mail}",
                "{config name=shopName}",
                "Ihre Leasing-Anfrage",
                file_get_contents(__DIR__ . '/Resources/mail/contract_submit/plain.txt'),
                file_get_contents(__DIR__ . '/Resources/mail/contract_submit/html.txt'),
                1,
                '',
                2,
                '',
                0
            ]
        );
    }

    public static function getSubscribedEvents()
    {
        return [
            'Theme_Inheritance_Template_Directories_Collected' => 'onCollectTemplateDir',
            'Enlight_Controller_Action_PostDispatchSecure_Frontend_Detail' => 'onPostDispatchSecure',
            'Theme_Compiler_Collect_Plugin_Less' => 'onCollectLessFiles',
            'Enlight_Controller_Dispatcher_ControllerPath_Backend_Albis' => 'onGetControllerPathBackend',
        ];
    }

    public function onGetControllerPathBackend(\Enlight_Event_EventArgs $args)
    {
        return __DIR__ . '/Controllers/Backend/Albis.php';
    }

    public function onCollectLessFiles(Enlight_Event_EventArgs $args)
    {
        $less = new LessDefinition(
            [],
            [__DIR__ . '/Resources/views/frontend/_public/src/less/all.less'],
            __DIR__
        );

        return new ArrayCollection([$less]);
    }

    public function onPostDispatchSecure(\Enlight_Event_EventArgs $args)
    {
        /** @var \Shopware_Controllers_Frontend_Detail $subject */
        $subject = $args->getSubject();

        $articlePrice = (float)$subject->View()->sArticle['price_numeric'] / ((100 + (float)$subject->View()->sArticle['tax']) / 100);
        $contractType = (int)$subject->View()->sArticle['albis_contract_type'];

        if ($contractType <= 0) {
            $contractType = 1;
        }

        try {
            $rates = $this->container->get('albis_payment5.components.api.albis_client')->getLeasingRates(
                $articlePrice,
                0.00,
                $contractType
            );
        } catch (\Exception $e) {
            $rates = [];
        }

        if (isset($rates['raten'])) {
            $rate = array_values($rates['raten']);
            $subject->View()->rate = end($rate);
        }

        $subject->View()->assign('articlePriceNet', $articlePrice);
    }

    /**
     * @param \Enlight_Event_EventArgs $args
     * @return void
     */
    public function onCollectTemplateDir(\Enlight_Event_EventArgs $args)
    {
        $dirs = $args->getReturn();
        $dirs[] = __DIR__ . '/Resources/views';
        $args->setReturn($dirs);
    }

    /**
     * @param UninstallContext $context
     */
    public function uninstall(UninstallContext $context)
    {
        $this->setActiveFlag($context->getPlugin()->getPayments(), false);

        $service = $this->container->get('shopware_attribute.crud_service');

        $service->delete('s_articles_attributes', 'albis_contract_type');

        $metaDataCache = Shopware()->Models()->getConfiguration()->getMetadataCacheImpl();
        $metaDataCache->deleteAll();

        Shopware()->Models()->generateAttributeModels(['s_articles_attributes']);
    }

    /**
     * @param DeactivateContext $context
     */
    public function deactivate(DeactivateContext $context)
    {
        $this->setActiveFlag($context->getPlugin()->getPayments(), false);
    }

    /**
     * @param ActivateContext $context
     */
    public function activate(ActivateContext $context)
    {
        $this->setActiveFlag($context->getPlugin()->getPayments(), true);
    }

    /**
     * @param \Doctrine\Common\Collections\ArrayCollection<\Shopware\Models\Payment\Payment> $payments
     * @param $active bool
     */
    private function setActiveFlag($payments, bool $active)
    {
        $em = $this->container->get('models');

        foreach ($payments as $payment) {
            $payment->setActive($active);
        }
        if (isset($em)) {
            $em->flush();
        }
    }
}
