<?php

namespace AlbisPayment5\Components\Api;

use GuzzleHttp\Client;
use Shopware\Components\Plugin\Configuration\ReaderInterface;

class AlbisClient
{
    private array $config;

    private Client $client;

    public function __construct()
    {
        $shop = Shopware()->Container()->get('shop');
        $this->config = Shopware()->Container()->get(ReaderInterface::class)->getByPluginName(
            'AlbisPayment5',
            $shop->getId()
        );

        $this->client = new Client();
    }

    public function getLeasingRates(
        float $price,
        float $specialPayment = 0.00,
        int $contractType = 1
    ): array {
        $response = $this->client->post($this->config['endpoint'], [
            'json' => [
                'id' => $this->getRequestId(),
                'method' => 'getRate',
                'params' => [
                    $this->config['username'],
                    $this->config['password'],
                    [
                        'kaufpreis' => $price,
                        'mietsz' => $specialPayment,
                        'prodgrp' => $contractType === 1 ? $this->config['productGroup'] : $this->config['secondContractProductGroup'],
                        'vertragsart' => 1,
                        'zahlweise' => 2,
                        'provision' => $this->config['provision'],
                    ],
                ],
            ],
        ]);

        $data = json_decode($response->getBody()->getContents(), true);

        if ($data['error']) {
            throw new \RuntimeException($data['error']['message'], (int)$data['error']['message']);
        }

        return $data['result'];
    }

    public function getAlbisCompanyTypes(): array
    {
        $response = $this->client->post($this->config['endpoint'], [
            'json' => [
                'id' => $this->getRequestId(),
                'method' => 'getRechtsformen',
                'params' => [
                    $this->config['username'],
                    $this->config['password'],
                ],
            ],
        ]);

        $resultRaw = json_decode($response->getBody()->getContents(), true);
        if (isset($resultRaw['error'])) {
            return [];
        }

        return $resultRaw['result'];
    }

    public function sendLeaseApplication(array $payload, int $contractType = 1)
    {
        $payload['prodgrp'] = $contractType === 1 ? $this->config['productGroup'] : $this->config['secondContractProductGroup'];

        return $this->client->post($this->config['endpoint'], [
            'json' => [
                'id' => $this->getRequestId(),
                'method' => 'stelleAntrag',
                'params' => [
                    $contractType === 1 ? $this->config['username'] : $this->config['secondContractUsername'],
                    $contractType === 1 ? $this->config['password'] : $this->config['secondContractPassword'],
                    $payload,
                ],
            ],
        ]);
    }

    private function getRequestId()
    {
        return sha1(microtime());
    }
}
