<?php

declare(strict_types=1);

use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

class Shopware_Controllers_Backend_Albis extends \Shopware_Controllers_Backend_ExtJs
{
    public function indexAction()
    {

    }

    public function checkAction()
    {
        $config = $this->Request()->getParams();
        $endpoint = $config['endpoint'];
        $username = $config['username'];
        $password = $config['password'];
        $secondContractUsername = $config['secondContractUsername'];
        $secondContractPassword = $config['secondContractPassword'];

        if (!$endpoint || !$username || !$password) {
            $data = [
                'ACK' => 'Failed',
                'message' => 'Bitte prüfen Sie den Endpunkt und Ihre Zugangsdaten',
            ];
            $this->View()->assign($data);

            return;
        }

        $client = new Client();
        $secondClient = new Client();

        try {
            $response = $client->post($endpoint, [
                'json' => [
                    'id' => sha1(microtime()),
                    'method' => 'ping',
                    'params' => [
                        $username,
                        $password,
                    ],
                ],
            ]);
            $response = json_decode($response->getBody()->getContents(), true);

            $secondResponse = $secondClient->post($endpoint, [
                'json' => [
                    'id' => sha1(microtime()),
                    'method' => 'ping',
                    'params' => [
                        $secondContractUsername,
                        $secondContractPassword,
                    ],
                ],
            ]);
            $secondResponse = json_decode($secondResponse->getBody()->getContents(), true);
            if ($response['result'] === 'pong' && $secondResponse['result'] === 'pong') {
                $data = [
                    'ACK' => 'Success',
                    'message' => 'OK',
                ];
            } else {
                $data = [
                    'ACK' => 'Failed',
                    'message' => 'Bitte prüfen Sie die URL und Ihre Zugangsdaten <br>',
                ];
            }
        } catch (GuzzleException $e) {
            $data = [
                'code' => $e->getCode(),
                'message' => 'Bitte prüfen Sie die URL und Ihre Zugangsdaten <br>' . $e->getMessage(),
            ];
        }

        $shopwareRelease = $this->container->get('shopware.release');

        $data['shopware_version'] = $shopwareRelease->getVersion();
        $data['php_version'] = PHP_VERSION;

        $this->View()->assign($data);
    }
}
