<?php

use AlbisPayment5\Components\Api\AlbisClient;
use Shopware\Components\Plugin\Configuration\ReaderInterface;

class Shopware_Controllers_Frontend_Albispayment extends \Shopware_Controllers_Frontend_Payment
{
    private array $config;

    /**
     * @var AlbisClient
     */
    private $albisClient;

    public function __construct()
    {
        parent::__construct();
        $shop = Shopware()->Container()->get('shop');
        $this->config = Shopware()->Container()->get(ReaderInterface::class)->getByPluginName(
            'AlbisPayment5',
            $shop->getId()
        );
        $this->albisClient = Shopware()->Container()->get('albis_payment5.components.api.albis_client');
    }

    public function indexAction()
    {
        $basket = $this->getBasket();
        $price = $basket['AmountNetNumeric'];
        $user = $this->getUser();

        $basketData = $this->getBasket();
        $contractType = $this->getContractType($basketData);

        try {
            $rates = $this->albisClient->getLeasingRates(
                (float)$price,
                (float)$this->Request()->get('prePayment'),
                $contractType
            );
            $prePaymentError = false;
        } catch (Exception $e) {
            try {
                $rates = $this->albisClient->getLeasingRates(
                    (float)$price,
                    0,
                    $contractType
                );
            } catch (Exception $e) {
            }
            $prePaymentError = true;
        }

        $this->view->assign(
            [
                'albisRates' => $rates,
                'albisCompanyTypes' => $this->albisClient->getAlbisCompanyTypes(),
                'albisUser' => $user,
                'prePaymentError' => $prePaymentError,
                'email' => $user['additional']['user']['email'],
                'phoneNumber' => $user['billingaddress']['phone'],
                'city' => $user['billingaddress']['city'],
                'zipCode' => $user['billingaddress']['zipcode'],
                'street' => $user['billingaddress']['street'],
                'company' => $user['billingaddress']['company'] ?? '',
                'prePayment' => $this->Request()->get('prePayment', ''),
                'basketToken' => $this->persistBasket(),
                'albisErrorPrice' => $price * 0.64,
            ]
        );
    }

    public function requestAction()
    {
        $signature = $this->Request()->getParam('basketToken');
        $prePayment = (float) $this->Request()->get('prePayment', 0.00);

        $basketData = $this->getBasket();

        $rate = explode('|', $this->Request()->get('rate'));
        $post = $this->Request()->getParams();

        $requestId = $this->getRequestId();

        $userData = $this->getUser();

        $response = $this->albisClient->sendLeaseApplication(
            [
                'objekt' => 'Shopware Bestellung - ' . $requestId,
                'kaufpreis' => $basketData['AmountNetNumeric'],
                'mietsz' => $prePayment, // Sonderzahlung
                'laufzeit' => $rate[1],
                'rate' => $rate[0],
                'leasingnehmer' => [
                    'name' => $post['company'],
                    'strasse' => $post['street'],
                    'plz' => $post['zip_code'],
                    'ort' => $post['city'],
                    'geschaeftsfuehrer' => [
                        'anrede' => $post['md_salutation'] ?: null,
                        'vorname' => $post['md_first_name'] ?: null,
                        'nachname' => $post['md_last_name'] ?: null,
                        'strasse' => $post['md_street'] ?: null,
                        'plz' => $post['md_zip_code'] ?: null,
                        'ort' => $post['md_city'] ?: null,
                        'gebdat' => $post['md_dob_year'] . '-' .
                            sprintf('%02d', $post['md_dob_month']) . '-' .
                            sprintf('%02d', $post['md_dob_day']),
                    ],
                    'rechtsform' => $post['legal_form'],
                    'telefon' => $post['phone_number'],
                    'email' => $post['email'],
                ],
                'provision' => $this->config['provision'],
                'vertragsart' => 1,
                'prodgrp' => 1,
                'zahlweise' => 2,
                'iban' => $post['iban'],
                'vertrag_an_ln' => true,
            ],
            $this->getContractType($basketData)
        );

        $response = json_decode($response->getBody()->getContents(), true);

        if ($response['error']) {
            $this->view->assign(
                [
                    'errorMessage' => trim(str_replace('server error:', '', $response['error']['message'])),
                    'ap' => $post,
                    'laufzeit' => $rate[1],
                ]
            );
            $this->indexAction();
        } else {
            $basket = $this->loadBasketFromSignature($signature);
            $this->verifyBasketSignature($signature, $basket);
            $orderNumber = $this->saveOrder($requestId, $signature, 0);

            $context = [
                'rate' => $rate,
                'ap' => $post,
                'basket' => $basketData,
                'user' => $userData,
                'prePayment' => $prePayment,
                'albisId' => $requestId,
                'orderNumber' => $orderNumber
            ];

            $mail = Shopware()->TemplateMail()->createMail('sORDERAlbis', $context);


            $mail->addTo($userData['additional']['user']['email']);

            if (!Shopware()->Config()->get('sNO_ORDER_MAIL')) {
                $mail->addBcc(Shopware()->Config()->get('sMAIL'));
            }

            try {
                $mail->send();
            } catch(Exception $exception) {
                // nth
            }

            $this->redirect(['controller' => 'checkout', 'action' => 'finish']);
        }
    }

    private function getContractType($basketData): int
    {
        foreach ($basketData['content'] as $position) {
            if ($position['additional_details']['attributes']['core']['albis_contract_type'] != 2) {
                return 1;
            }
        }

        return 2;
    }

    private function getRequestId()
    {
        return sha1(microtime());
    }
}
